@@ -20,6 +20,8 @@
 #include <QObject>
 #include <QTimer>
 
+#include <chrono>
+
 #include "kglobalaccel_export.h"
 
 class Component;
@@ -159,11 +161,28 @@ private:
     // returns true if the key was handled
     bool keyPressed(int keyQt);
     bool keyReleased(int keyQt);
+    bool pointerPressed(Qt::MouseButtons pointerButtons);
+    bool axisTriggered(int axis);
+
+    bool processKey(int keyQt);
 
     QHash<QKeySequence, GlobalShortcut *> _active_keys;
     QKeySequence _active_sequence;
     QHash<int, int> _keys_count;
 
+    Qt::KeyboardModifiers m_currentModifiers;
+    // State machine:
+    // Any -> PressingModifierOnly when a modifier is pressed
+    // PressingModifierOnly -> ReleasingModifierOnly when modifier keys are released
+    // ReleasingModifierOnly -> Normal when other keys are pressed
+    // ReleasingModifierOnly -> Normal when all keys are released, and when a non-modifier key is pressed
+    // Modifier-only shortcuts are triggered in ReleasingModifierOnly state, if all keys are released
+    // and the time since the first release is less than some threshold
+    enum { Normal, PressingModifierOnly, ReleasingModifierOnly } m_state = Normal;
+    // For use in modifier only shortcuts
+    Qt::KeyboardModifiers m_modifierOnlyModifiers;
+    std::chrono::nanoseconds m_modifierFirstReleaseTime;
+
     using ComponentVec = std::vector<ComponentPtr>;
     ComponentVec m_components;
     ComponentVec::const_iterator findByName(const QString &name) const
